# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel
from .dynamic_emergency_endpoint import DynamicEmergencyEndpoint

__all__ = ["DynamicEmergencyEndpointRetrieveResponse"]


class DynamicEmergencyEndpointRetrieveResponse(BaseModel):
    data: Optional[DynamicEmergencyEndpoint] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from telnyx import Telnyx, AsyncTelnyx
from tests.utils import assert_matches_type
from telnyx._utils import parse_datetime
from telnyx.types.reports import CdrUsageReportFetchSyncResponse

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestCdrUsageReports:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_fetch_sync(self, client: Telnyx) -> None:
        cdr_usage_report = client.reports.cdr_usage_reports.fetch_sync(
            aggregation_type="NO_AGGREGATION",
            product_breakdown="NO_BREAKDOWN",
        )
        assert_matches_type(CdrUsageReportFetchSyncResponse, cdr_usage_report, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_fetch_sync_with_all_params(self, client: Telnyx) -> None:
        cdr_usage_report = client.reports.cdr_usage_reports.fetch_sync(
            aggregation_type="NO_AGGREGATION",
            product_breakdown="NO_BREAKDOWN",
            connections=[1234567890123],
            end_date=parse_datetime("2020-07-01T00:00:00-06:00"),
            start_date=parse_datetime("2020-07-01T00:00:00-06:00"),
        )
        assert_matches_type(CdrUsageReportFetchSyncResponse, cdr_usage_report, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_fetch_sync(self, client: Telnyx) -> None:
        response = client.reports.cdr_usage_reports.with_raw_response.fetch_sync(
            aggregation_type="NO_AGGREGATION",
            product_breakdown="NO_BREAKDOWN",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        cdr_usage_report = response.parse()
        assert_matches_type(CdrUsageReportFetchSyncResponse, cdr_usage_report, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_fetch_sync(self, client: Telnyx) -> None:
        with client.reports.cdr_usage_reports.with_streaming_response.fetch_sync(
            aggregation_type="NO_AGGREGATION",
            product_breakdown="NO_BREAKDOWN",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            cdr_usage_report = response.parse()
            assert_matches_type(CdrUsageReportFetchSyncResponse, cdr_usage_report, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncCdrUsageReports:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_fetch_sync(self, async_client: AsyncTelnyx) -> None:
        cdr_usage_report = await async_client.reports.cdr_usage_reports.fetch_sync(
            aggregation_type="NO_AGGREGATION",
            product_breakdown="NO_BREAKDOWN",
        )
        assert_matches_type(CdrUsageReportFetchSyncResponse, cdr_usage_report, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_fetch_sync_with_all_params(self, async_client: AsyncTelnyx) -> None:
        cdr_usage_report = await async_client.reports.cdr_usage_reports.fetch_sync(
            aggregation_type="NO_AGGREGATION",
            product_breakdown="NO_BREAKDOWN",
            connections=[1234567890123],
            end_date=parse_datetime("2020-07-01T00:00:00-06:00"),
            start_date=parse_datetime("2020-07-01T00:00:00-06:00"),
        )
        assert_matches_type(CdrUsageReportFetchSyncResponse, cdr_usage_report, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_fetch_sync(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.reports.cdr_usage_reports.with_raw_response.fetch_sync(
            aggregation_type="NO_AGGREGATION",
            product_breakdown="NO_BREAKDOWN",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        cdr_usage_report = await response.parse()
        assert_matches_type(CdrUsageReportFetchSyncResponse, cdr_usage_report, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_fetch_sync(self, async_client: AsyncTelnyx) -> None:
        async with async_client.reports.cdr_usage_reports.with_streaming_response.fetch_sync(
            aggregation_type="NO_AGGREGATION",
            product_breakdown="NO_BREAKDOWN",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            cdr_usage_report = await response.parse()
            assert_matches_type(CdrUsageReportFetchSyncResponse, cdr_usage_report, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["ActionRecordStartParams"]


class ActionRecordStartParams(TypedDict, total=False):
    format: Required[Literal["wav", "mp3"]]
    """The audio file format used when storing the conference recording.

    Can be either `mp3` or `wav`.
    """

    command_id: str
    """Use this field to avoid duplicate commands.

    Telnyx will ignore any command with the same `command_id` for the same
    `conference_id`.
    """

    custom_file_name: str
    """The custom recording file name to be used instead of the default `call_leg_id`.

    Telnyx will still add a Unix timestamp suffix.
    """

    play_beep: bool
    """If enabled, a beep sound will be played at the start of a recording."""

    region: Literal["Australia", "Europe", "Middle East", "US"]
    """Region where the conference data is located.

    Defaults to the region defined in user's data locality settings (Europe or US).
    """

    trim: Literal["trim-silence"]
    """
    When set to `trim-silence`, silence will be removed from the beginning and end
    of the recording.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Literal

import httpx

from ..types import document_list_params, document_update_params, document_upload_params, document_upload_json_params
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    BinaryAPIResponse,
    AsyncBinaryAPIResponse,
    StreamedBinaryAPIResponse,
    AsyncStreamedBinaryAPIResponse,
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    to_custom_raw_response_wrapper,
    async_to_streamed_response_wrapper,
    to_custom_streamed_response_wrapper,
    async_to_custom_raw_response_wrapper,
    async_to_custom_streamed_response_wrapper,
)
from ..pagination import SyncDefaultFlatPagination, AsyncDefaultFlatPagination
from .._base_client import AsyncPaginator, make_request_options
from ..types.doc_service_document import DocServiceDocument
from ..types.document_delete_response import DocumentDeleteResponse
from ..types.document_update_response import DocumentUpdateResponse
from ..types.document_upload_response import DocumentUploadResponse
from ..types.document_retrieve_response import DocumentRetrieveResponse
from ..types.document_upload_json_response import DocumentUploadJsonResponse
from ..types.document_generate_download_link_response import DocumentGenerateDownloadLinkResponse

__all__ = ["DocumentsResource", "AsyncDocumentsResource"]


class DocumentsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> DocumentsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return DocumentsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> DocumentsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return DocumentsResourceWithStreamingResponse(self)

    def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DocumentRetrieveResponse:
        """
        Retrieve a document.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/documents/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DocumentRetrieveResponse,
        )

    def update(
        self,
        document_id: str,
        *,
        customer_reference: str | Omit = omit,
        filename: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DocumentUpdateResponse:
        """
        Update a document.

        Args:
          customer_reference: Optional reference string for customer tracking.

          filename: The filename of the document.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not document_id:
            raise ValueError(f"Expected a non-empty value for `document_id` but received {document_id!r}")
        return self._patch(
            f"/documents/{document_id}",
            body=maybe_transform(
                {
                    "customer_reference": customer_reference,
                    "filename": filename,
                },
                document_update_params.DocumentUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DocumentUpdateResponse,
        )

    def list(
        self,
        *,
        filter: document_list_params.Filter | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        sort: List[Literal["filename", "created_at", "updated_at", "-filename", "-created_at", "-updated_at"]]
        | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncDefaultFlatPagination[DocServiceDocument]:
        """
        List all documents ordered by created_at descending.

        Args:
          filter:
              Consolidated filter parameter for documents (deepObject style). Originally:
              filter[filename][contains], filter[customer_reference][eq],
              filter[customer_reference][in][], filter[created_at][gt], filter[created_at][lt]

          sort: Consolidated sort parameter for documents (deepObject style). Originally: sort[]

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/documents",
            page=SyncDefaultFlatPagination[DocServiceDocument],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter": filter,
                        "page_number": page_number,
                        "page_size": page_size,
                        "sort": sort,
                    },
                    document_list_params.DocumentListParams,
                ),
            ),
            model=DocServiceDocument,
        )

    def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DocumentDeleteResponse:
        """
        Delete a document.<br /><br />A document can only be deleted if it's not linked
        to a service. If it is linked to a service, it must be unlinked prior to
        deleting.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._delete(
            f"/documents/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DocumentDeleteResponse,
        )

    def download(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BinaryAPIResponse:
        """
        Download a document.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Accept": "application/octet-stream", **(extra_headers or {})}
        return self._get(
            f"/documents/{id}/download",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BinaryAPIResponse,
        )

    def generate_download_link(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DocumentGenerateDownloadLinkResponse:
        """
        Generates a temporary pre-signed URL that can be used to download the document
        directly from the storage backend without authentication.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/documents/{id}/download_link",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DocumentGenerateDownloadLinkResponse,
        )

    def upload(
        self,
        *,
        document: document_upload_params.Document,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DocumentUploadResponse:
        """
        Upload a document.<br /><br />Uploaded files must be linked to a service within
        30 minutes or they will be automatically deleted.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/documents?content-type=multipart",
            body=maybe_transform(document, document_upload_params.DocumentUploadParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DocumentUploadResponse,
        )

    def upload_json(
        self,
        *,
        document: document_upload_json_params.Document,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DocumentUploadJsonResponse:
        """
        Upload a document.<br /><br />Uploaded files must be linked to a service within
        30 minutes or they will be automatically deleted.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/documents",
            body=maybe_transform(document, document_upload_json_params.DocumentUploadJsonParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DocumentUploadJsonResponse,
        )


class AsyncDocumentsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncDocumentsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncDocumentsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncDocumentsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncDocumentsResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DocumentRetrieveResponse:
        """
        Retrieve a document.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/documents/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DocumentRetrieveResponse,
        )

    async def update(
        self,
        document_id: str,
        *,
        customer_reference: str | Omit = omit,
        filename: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DocumentUpdateResponse:
        """
        Update a document.

        Args:
          customer_reference: Optional reference string for customer tracking.

          filename: The filename of the document.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not document_id:
            raise ValueError(f"Expected a non-empty value for `document_id` but received {document_id!r}")
        return await self._patch(
            f"/documents/{document_id}",
            body=await async_maybe_transform(
                {
                    "customer_reference": customer_reference,
                    "filename": filename,
                },
                document_update_params.DocumentUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DocumentUpdateResponse,
        )

    def list(
        self,
        *,
        filter: document_list_params.Filter | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        sort: List[Literal["filename", "created_at", "updated_at", "-filename", "-created_at", "-updated_at"]]
        | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[DocServiceDocument, AsyncDefaultFlatPagination[DocServiceDocument]]:
        """
        List all documents ordered by created_at descending.

        Args:
          filter:
              Consolidated filter parameter for documents (deepObject style). Originally:
              filter[filename][contains], filter[customer_reference][eq],
              filter[customer_reference][in][], filter[created_at][gt], filter[created_at][lt]

          sort: Consolidated sort parameter for documents (deepObject style). Originally: sort[]

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/documents",
            page=AsyncDefaultFlatPagination[DocServiceDocument],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter": filter,
                        "page_number": page_number,
                        "page_size": page_size,
                        "sort": sort,
                    },
                    document_list_params.DocumentListParams,
                ),
            ),
            model=DocServiceDocument,
        )

    async def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DocumentDeleteResponse:
        """
        Delete a document.<br /><br />A document can only be deleted if it's not linked
        to a service. If it is linked to a service, it must be unlinked prior to
        deleting.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._delete(
            f"/documents/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DocumentDeleteResponse,
        )

    async def download(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncBinaryAPIResponse:
        """
        Download a document.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Accept": "application/octet-stream", **(extra_headers or {})}
        return await self._get(
            f"/documents/{id}/download",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AsyncBinaryAPIResponse,
        )

    async def generate_download_link(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DocumentGenerateDownloadLinkResponse:
        """
        Generates a temporary pre-signed URL that can be used to download the document
        directly from the storage backend without authentication.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/documents/{id}/download_link",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DocumentGenerateDownloadLinkResponse,
        )

    async def upload(
        self,
        *,
        document: document_upload_params.Document,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DocumentUploadResponse:
        """
        Upload a document.<br /><br />Uploaded files must be linked to a service within
        30 minutes or they will be automatically deleted.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/documents?content-type=multipart",
            body=await async_maybe_transform(document, document_upload_params.DocumentUploadParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DocumentUploadResponse,
        )

    async def upload_json(
        self,
        *,
        document: document_upload_json_params.Document,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DocumentUploadJsonResponse:
        """
        Upload a document.<br /><br />Uploaded files must be linked to a service within
        30 minutes or they will be automatically deleted.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/documents",
            body=await async_maybe_transform(document, document_upload_json_params.DocumentUploadJsonParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DocumentUploadJsonResponse,
        )


class DocumentsResourceWithRawResponse:
    def __init__(self, documents: DocumentsResource) -> None:
        self._documents = documents

        self.retrieve = to_raw_response_wrapper(
            documents.retrieve,
        )
        self.update = to_raw_response_wrapper(
            documents.update,
        )
        self.list = to_raw_response_wrapper(
            documents.list,
        )
        self.delete = to_raw_response_wrapper(
            documents.delete,
        )
        self.download = to_custom_raw_response_wrapper(
            documents.download,
            BinaryAPIResponse,
        )
        self.generate_download_link = to_raw_response_wrapper(
            documents.generate_download_link,
        )
        self.upload = to_raw_response_wrapper(
            documents.upload,
        )
        self.upload_json = to_raw_response_wrapper(
            documents.upload_json,
        )


class AsyncDocumentsResourceWithRawResponse:
    def __init__(self, documents: AsyncDocumentsResource) -> None:
        self._documents = documents

        self.retrieve = async_to_raw_response_wrapper(
            documents.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            documents.update,
        )
        self.list = async_to_raw_response_wrapper(
            documents.list,
        )
        self.delete = async_to_raw_response_wrapper(
            documents.delete,
        )
        self.download = async_to_custom_raw_response_wrapper(
            documents.download,
            AsyncBinaryAPIResponse,
        )
        self.generate_download_link = async_to_raw_response_wrapper(
            documents.generate_download_link,
        )
        self.upload = async_to_raw_response_wrapper(
            documents.upload,
        )
        self.upload_json = async_to_raw_response_wrapper(
            documents.upload_json,
        )


class DocumentsResourceWithStreamingResponse:
    def __init__(self, documents: DocumentsResource) -> None:
        self._documents = documents

        self.retrieve = to_streamed_response_wrapper(
            documents.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            documents.update,
        )
        self.list = to_streamed_response_wrapper(
            documents.list,
        )
        self.delete = to_streamed_response_wrapper(
            documents.delete,
        )
        self.download = to_custom_streamed_response_wrapper(
            documents.download,
            StreamedBinaryAPIResponse,
        )
        self.generate_download_link = to_streamed_response_wrapper(
            documents.generate_download_link,
        )
        self.upload = to_streamed_response_wrapper(
            documents.upload,
        )
        self.upload_json = to_streamed_response_wrapper(
            documents.upload_json,
        )


class AsyncDocumentsResourceWithStreamingResponse:
    def __init__(self, documents: AsyncDocumentsResource) -> None:
        self._documents = documents

        self.retrieve = async_to_streamed_response_wrapper(
            documents.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            documents.update,
        )
        self.list = async_to_streamed_response_wrapper(
            documents.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            documents.delete,
        )
        self.download = async_to_custom_streamed_response_wrapper(
            documents.download,
            AsyncStreamedBinaryAPIResponse,
        )
        self.generate_download_link = async_to_streamed_response_wrapper(
            documents.generate_download_link,
        )
        self.upload = async_to_streamed_response_wrapper(
            documents.upload,
        )
        self.upload_json = async_to_streamed_response_wrapper(
            documents.upload_json,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["VerifiedNumberCreateParams"]


class VerifiedNumberCreateParams(TypedDict, total=False):
    phone_number: Required[str]

    verification_method: Required[Literal["sms", "call"]]
    """Verification method."""

    extension: str
    """Optional DTMF extension sequence to dial after the call is answered.

    This parameter enables verification of phone numbers behind IVR systems that
    require extension dialing. Valid characters: digits 0-9, letters A-D, symbols \\**
    and #. Pauses: w = 0.5 second pause, W = 1 second pause. Maximum length: 50
    characters. Only works with 'call' verification method.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel
from .fqdn_connection import FqdnConnection

__all__ = ["FqdnConnectionCreateResponse"]


class FqdnConnectionCreateResponse(BaseModel):
    data: Optional[FqdnConnection] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["RcGenerateDeeplinkParams"]


class RcGenerateDeeplinkParams(TypedDict, total=False):
    body: str
    """Pre-filled message body (URL encoded)"""

    phone_number: str
    """Phone number in E164 format (URL encoded)"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Literal, TypedDict

from .audio_visualizer_config_param import AudioVisualizerConfigParam

__all__ = ["WidgetSettingsParam"]


class WidgetSettingsParam(TypedDict, total=False):
    """Configuration settings for the assistant's web widget."""

    agent_thinking_text: str
    """Text displayed while the agent is processing."""

    audio_visualizer_config: AudioVisualizerConfigParam

    default_state: Literal["expanded", "collapsed"]
    """The default state of the widget."""

    give_feedback_url: Optional[str]
    """URL for users to give feedback."""

    logo_icon_url: Optional[str]
    """URL to a custom logo icon for the widget."""

    position: Literal["fixed", "static"]
    """The positioning style for the widget."""

    report_issue_url: Optional[str]
    """URL for users to report issues."""

    speak_to_interrupt_text: str
    """Text prompting users to speak to interrupt."""

    start_call_text: str
    """Custom text displayed on the start call button."""

    theme: Literal["light", "dark"]
    """The visual theme for the widget."""

    view_history_url: Optional[str]
    """URL to view conversation history."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from .._models import BaseModel

__all__ = ["PortabilityCheckRunResponse", "Data"]


class Data(BaseModel):
    fast_portable: Optional[bool] = None
    """Indicates whether this phone number is FastPort eligible"""

    not_portable_reason: Optional[str] = None
    """If this phone number is not portable, explains why.

    Empty string if the number is portable.
    """

    phone_number: Optional[str] = None
    """The +E.164 formatted phone number this result is about"""

    portable: Optional[bool] = None
    """Indicates whether this phone number is portable"""

    record_type: Optional[str] = None
    """Identifies the type of the resource."""


class PortabilityCheckRunResponse(BaseModel):
    data: Optional[List[Data]] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel
from .conference_participant_left import ConferenceParticipantLeft

__all__ = ["ConferenceParticipantLeftWebhookEvent"]


class ConferenceParticipantLeftWebhookEvent(BaseModel):
    data: Optional[ConferenceParticipantLeft] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Union
from typing_extensions import Literal, Annotated, TypedDict

from ..._types import SequenceNotStr
from ..._utils import PropertyInfo

__all__ = ["PhoneNumberBlockListParams", "Filter", "Sort"]


class PhoneNumberBlockListParams(TypedDict, total=False):
    filter: Filter
    """Consolidated filter parameter (deepObject style).

    Originally: filter[porting_order_id], filter[support_key], filter[status],
    filter[phone_number], filter[activation_status], filter[portability_status]
    """

    page_number: Annotated[int, PropertyInfo(alias="page[number]")]

    page_size: Annotated[int, PropertyInfo(alias="page[size]")]

    sort: Sort
    """Consolidated sort parameter (deepObject style). Originally: sort[value]"""


class Filter(TypedDict, total=False):
    """Consolidated filter parameter (deepObject style).

    Originally: filter[porting_order_id], filter[support_key], filter[status], filter[phone_number], filter[activation_status], filter[portability_status]
    """

    activation_status: Literal[
        "New",
        "Pending",
        "Conflict",
        "Cancel Pending",
        "Failed",
        "Concurred",
        "Activate RDY",
        "Disconnect Pending",
        "Concurrence Sent",
        "Old",
        "Sending",
        "Active",
        "Cancelled",
    ]
    """Filter results by activation status"""

    phone_number: SequenceNotStr[str]
    """Filter results by a list of phone numbers"""

    portability_status: Literal["pending", "confirmed", "provisional"]
    """Filter results by portability status"""

    porting_order_id: SequenceNotStr[str]
    """Filter results by a list of porting order ids"""

    status: Union[
        Literal[
            "draft",
            "in-process",
            "submitted",
            "exception",
            "foc-date-confirmed",
            "cancel-pending",
            "ported",
            "cancelled",
        ],
        List[
            Literal[
                "draft",
                "in-process",
                "submitted",
                "exception",
                "foc-date-confirmed",
                "cancel-pending",
                "ported",
                "cancelled",
            ]
        ],
    ]
    """Filter porting orders by status(es).

    Originally: filter[status], filter[status][in][]
    """

    support_key: Union[str, SequenceNotStr[str]]
    """Filter results by support key(s).

    Originally: filter[support_key][eq], filter[support_key][in][]
    """


class Sort(TypedDict, total=False):
    """Consolidated sort parameter (deepObject style). Originally: sort[value]"""

    value: Literal["-created_at", "created_at"]
    """Specifies the sort order for results.

    If not given, results are sorted by created_at in descending order
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["SimCardDataUsageNotificationCreateParams", "Threshold"]


class SimCardDataUsageNotificationCreateParams(TypedDict, total=False):
    sim_card_id: Required[str]
    """The identification UUID of the related SIM card resource."""

    threshold: Required[Threshold]
    """Data usage threshold that will trigger the notification."""


class Threshold(TypedDict, total=False):
    """Data usage threshold that will trigger the notification."""

    amount: str

    unit: Literal["MB", "GB"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["SpeechToTextTranscribeParams"]


class SpeechToTextTranscribeParams(TypedDict, total=False):
    input_format: Required[Literal["mp3", "wav"]]
    """The format of input audio stream."""

    transcription_engine: Required[Literal["Azure", "Deepgram", "Google", "Telnyx"]]
    """The transcription engine to use for processing the audio stream."""

    interim_results: bool
    """Whether to receive interim transcription results."""

    language: str
    """The language spoken in the audio stream."""

    model: Literal[
        "fast",
        "deepgram/nova-2",
        "deepgram/nova-3",
        "latest_long",
        "latest_short",
        "command_and_search",
        "phone_call",
        "video",
        "default",
        "medical_conversation",
        "medical_dictation",
        "openai/whisper-tiny",
        "openai/whisper-large-v3-turbo",
    ]
    """The specific model to use within the selected transcription engine."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..types import (
    verify_profile_list_params,
    verify_profile_create_params,
    verify_profile_update_params,
    verify_profile_create_template_params,
    verify_profile_update_template_params,
)
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..pagination import SyncDefaultFlatPagination, AsyncDefaultFlatPagination
from .._base_client import AsyncPaginator, make_request_options
from ..types.verify_profile import VerifyProfile
from ..types.message_template import MessageTemplate
from ..types.verify_profile_data import VerifyProfileData
from ..types.verify_profile_retrieve_templates_response import VerifyProfileRetrieveTemplatesResponse

__all__ = ["VerifyProfilesResource", "AsyncVerifyProfilesResource"]


class VerifyProfilesResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> VerifyProfilesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return VerifyProfilesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> VerifyProfilesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return VerifyProfilesResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        name: str,
        call: verify_profile_create_params.Call | Omit = omit,
        flashcall: verify_profile_create_params.Flashcall | Omit = omit,
        language: str | Omit = omit,
        sms: verify_profile_create_params.SMS | Omit = omit,
        webhook_failover_url: str | Omit = omit,
        webhook_url: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VerifyProfileData:
        """
        Creates a new Verify profile to associate verifications with.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/verify_profiles",
            body=maybe_transform(
                {
                    "name": name,
                    "call": call,
                    "flashcall": flashcall,
                    "language": language,
                    "sms": sms,
                    "webhook_failover_url": webhook_failover_url,
                    "webhook_url": webhook_url,
                },
                verify_profile_create_params.VerifyProfileCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=VerifyProfileData,
        )

    def retrieve(
        self,
        verify_profile_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VerifyProfileData:
        """
        Gets a single Verify profile.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not verify_profile_id:
            raise ValueError(f"Expected a non-empty value for `verify_profile_id` but received {verify_profile_id!r}")
        return self._get(
            f"/verify_profiles/{verify_profile_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=VerifyProfileData,
        )

    def update(
        self,
        verify_profile_id: str,
        *,
        call: verify_profile_update_params.Call | Omit = omit,
        flashcall: verify_profile_update_params.Flashcall | Omit = omit,
        language: str | Omit = omit,
        name: str | Omit = omit,
        sms: verify_profile_update_params.SMS | Omit = omit,
        webhook_failover_url: str | Omit = omit,
        webhook_url: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VerifyProfileData:
        """
        Update Verify profile

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not verify_profile_id:
            raise ValueError(f"Expected a non-empty value for `verify_profile_id` but received {verify_profile_id!r}")
        return self._patch(
            f"/verify_profiles/{verify_profile_id}",
            body=maybe_transform(
                {
                    "call": call,
                    "flashcall": flashcall,
                    "language": language,
                    "name": name,
                    "sms": sms,
                    "webhook_failover_url": webhook_failover_url,
                    "webhook_url": webhook_url,
                },
                verify_profile_update_params.VerifyProfileUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=VerifyProfileData,
        )

    def list(
        self,
        *,
        filter: verify_profile_list_params.Filter | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncDefaultFlatPagination[VerifyProfile]:
        """
        Gets a paginated list of Verify profiles.

        Args:
          filter: Consolidated filter parameter (deepObject style). Originally: filter[name]

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/verify_profiles",
            page=SyncDefaultFlatPagination[VerifyProfile],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter": filter,
                        "page_number": page_number,
                        "page_size": page_size,
                    },
                    verify_profile_list_params.VerifyProfileListParams,
                ),
            ),
            model=VerifyProfile,
        )

    def delete(
        self,
        verify_profile_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VerifyProfileData:
        """
        Delete Verify profile

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not verify_profile_id:
            raise ValueError(f"Expected a non-empty value for `verify_profile_id` but received {verify_profile_id!r}")
        return self._delete(
            f"/verify_profiles/{verify_profile_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=VerifyProfileData,
        )

    def create_template(
        self,
        *,
        text: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MessageTemplate:
        """
        Create a new Verify profile message template.

        Args:
          text: The text content of the message template.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/verify_profiles/templates",
            body=maybe_transform(
                {"text": text}, verify_profile_create_template_params.VerifyProfileCreateTemplateParams
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MessageTemplate,
        )

    def retrieve_templates(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VerifyProfileRetrieveTemplatesResponse:
        """List all Verify profile message templates."""
        return self._get(
            "/verify_profiles/templates",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=VerifyProfileRetrieveTemplatesResponse,
        )

    def update_template(
        self,
        template_id: str,
        *,
        text: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MessageTemplate:
        """
        Update an existing Verify profile message template.

        Args:
          text: The text content of the message template.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not template_id:
            raise ValueError(f"Expected a non-empty value for `template_id` but received {template_id!r}")
        return self._patch(
            f"/verify_profiles/templates/{template_id}",
            body=maybe_transform(
                {"text": text}, verify_profile_update_template_params.VerifyProfileUpdateTemplateParams
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MessageTemplate,
        )


class AsyncVerifyProfilesResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncVerifyProfilesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncVerifyProfilesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncVerifyProfilesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncVerifyProfilesResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        name: str,
        call: verify_profile_create_params.Call | Omit = omit,
        flashcall: verify_profile_create_params.Flashcall | Omit = omit,
        language: str | Omit = omit,
        sms: verify_profile_create_params.SMS | Omit = omit,
        webhook_failover_url: str | Omit = omit,
        webhook_url: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VerifyProfileData:
        """
        Creates a new Verify profile to associate verifications with.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/verify_profiles",
            body=await async_maybe_transform(
                {
                    "name": name,
                    "call": call,
                    "flashcall": flashcall,
                    "language": language,
                    "sms": sms,
                    "webhook_failover_url": webhook_failover_url,
                    "webhook_url": webhook_url,
                },
                verify_profile_create_params.VerifyProfileCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=VerifyProfileData,
        )

    async def retrieve(
        self,
        verify_profile_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VerifyProfileData:
        """
        Gets a single Verify profile.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not verify_profile_id:
            raise ValueError(f"Expected a non-empty value for `verify_profile_id` but received {verify_profile_id!r}")
        return await self._get(
            f"/verify_profiles/{verify_profile_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=VerifyProfileData,
        )

    async def update(
        self,
        verify_profile_id: str,
        *,
        call: verify_profile_update_params.Call | Omit = omit,
        flashcall: verify_profile_update_params.Flashcall | Omit = omit,
        language: str | Omit = omit,
        name: str | Omit = omit,
        sms: verify_profile_update_params.SMS | Omit = omit,
        webhook_failover_url: str | Omit = omit,
        webhook_url: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VerifyProfileData:
        """
        Update Verify profile

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not verify_profile_id:
            raise ValueError(f"Expected a non-empty value for `verify_profile_id` but received {verify_profile_id!r}")
        return await self._patch(
            f"/verify_profiles/{verify_profile_id}",
            body=await async_maybe_transform(
                {
                    "call": call,
                    "flashcall": flashcall,
                    "language": language,
                    "name": name,
                    "sms": sms,
                    "webhook_failover_url": webhook_failover_url,
                    "webhook_url": webhook_url,
                },
                verify_profile_update_params.VerifyProfileUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=VerifyProfileData,
        )

    def list(
        self,
        *,
        filter: verify_profile_list_params.Filter | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[VerifyProfile, AsyncDefaultFlatPagination[VerifyProfile]]:
        """
        Gets a paginated list of Verify profiles.

        Args:
          filter: Consolidated filter parameter (deepObject style). Originally: filter[name]

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/verify_profiles",
            page=AsyncDefaultFlatPagination[VerifyProfile],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter": filter,
                        "page_number": page_number,
                        "page_size": page_size,
                    },
                    verify_profile_list_params.VerifyProfileListParams,
                ),
            ),
            model=VerifyProfile,
        )

    async def delete(
        self,
        verify_profile_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VerifyProfileData:
        """
        Delete Verify profile

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not verify_profile_id:
            raise ValueError(f"Expected a non-empty value for `verify_profile_id` but received {verify_profile_id!r}")
        return await self._delete(
            f"/verify_profiles/{verify_profile_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=VerifyProfileData,
        )

    async def create_template(
        self,
        *,
        text: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MessageTemplate:
        """
        Create a new Verify profile message template.

        Args:
          text: The text content of the message template.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/verify_profiles/templates",
            body=await async_maybe_transform(
                {"text": text}, verify_profile_create_template_params.VerifyProfileCreateTemplateParams
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MessageTemplate,
        )

    async def retrieve_templates(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VerifyProfileRetrieveTemplatesResponse:
        """List all Verify profile message templates."""
        return await self._get(
            "/verify_profiles/templates",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=VerifyProfileRetrieveTemplatesResponse,
        )

    async def update_template(
        self,
        template_id: str,
        *,
        text: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MessageTemplate:
        """
        Update an existing Verify profile message template.

        Args:
          text: The text content of the message template.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not template_id:
            raise ValueError(f"Expected a non-empty value for `template_id` but received {template_id!r}")
        return await self._patch(
            f"/verify_profiles/templates/{template_id}",
            body=await async_maybe_transform(
                {"text": text}, verify_profile_update_template_params.VerifyProfileUpdateTemplateParams
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MessageTemplate,
        )


class VerifyProfilesResourceWithRawResponse:
    def __init__(self, verify_profiles: VerifyProfilesResource) -> None:
        self._verify_profiles = verify_profiles

        self.create = to_raw_response_wrapper(
            verify_profiles.create,
        )
        self.retrieve = to_raw_response_wrapper(
            verify_profiles.retrieve,
        )
        self.update = to_raw_response_wrapper(
            verify_profiles.update,
        )
        self.list = to_raw_response_wrapper(
            verify_profiles.list,
        )
        self.delete = to_raw_response_wrapper(
            verify_profiles.delete,
        )
        self.create_template = to_raw_response_wrapper(
            verify_profiles.create_template,
        )
        self.retrieve_templates = to_raw_response_wrapper(
            verify_profiles.retrieve_templates,
        )
        self.update_template = to_raw_response_wrapper(
            verify_profiles.update_template,
        )


class AsyncVerifyProfilesResourceWithRawResponse:
    def __init__(self, verify_profiles: AsyncVerifyProfilesResource) -> None:
        self._verify_profiles = verify_profiles

        self.create = async_to_raw_response_wrapper(
            verify_profiles.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            verify_profiles.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            verify_profiles.update,
        )
        self.list = async_to_raw_response_wrapper(
            verify_profiles.list,
        )
        self.delete = async_to_raw_response_wrapper(
            verify_profiles.delete,
        )
        self.create_template = async_to_raw_response_wrapper(
            verify_profiles.create_template,
        )
        self.retrieve_templates = async_to_raw_response_wrapper(
            verify_profiles.retrieve_templates,
        )
        self.update_template = async_to_raw_response_wrapper(
            verify_profiles.update_template,
        )


class VerifyProfilesResourceWithStreamingResponse:
    def __init__(self, verify_profiles: VerifyProfilesResource) -> None:
        self._verify_profiles = verify_profiles

        self.create = to_streamed_response_wrapper(
            verify_profiles.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            verify_profiles.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            verify_profiles.update,
        )
        self.list = to_streamed_response_wrapper(
            verify_profiles.list,
        )
        self.delete = to_streamed_response_wrapper(
            verify_profiles.delete,
        )
        self.create_template = to_streamed_response_wrapper(
            verify_profiles.create_template,
        )
        self.retrieve_templates = to_streamed_response_wrapper(
            verify_profiles.retrieve_templates,
        )
        self.update_template = to_streamed_response_wrapper(
            verify_profiles.update_template,
        )


class AsyncVerifyProfilesResourceWithStreamingResponse:
    def __init__(self, verify_profiles: AsyncVerifyProfilesResource) -> None:
        self._verify_profiles = verify_profiles

        self.create = async_to_streamed_response_wrapper(
            verify_profiles.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            verify_profiles.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            verify_profiles.update,
        )
        self.list = async_to_streamed_response_wrapper(
            verify_profiles.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            verify_profiles.delete,
        )
        self.create_template = async_to_streamed_response_wrapper(
            verify_profiles.create_template,
        )
        self.retrieve_templates = async_to_streamed_response_wrapper(
            verify_profiles.retrieve_templates,
        )
        self.update_template = async_to_streamed_response_wrapper(
            verify_profiles.update_template,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel
from .sim_card_action import SimCardAction

__all__ = ["ActionSetStandbyResponse"]


class ActionSetStandbyResponse(BaseModel):
    data: Optional[SimCardAction] = None
    """This object represents a SIM card action.

    It allows tracking the current status of an operation that impacts the SIM card.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...pagination import SyncDefaultFlatPagination, AsyncDefaultFlatPagination
from ..._base_client import AsyncPaginator, make_request_options
from ...types.external_connections import phone_number_list_params, phone_number_update_params
from ...types.external_connections.phone_number_update_response import PhoneNumberUpdateResponse
from ...types.external_connections.phone_number_retrieve_response import PhoneNumberRetrieveResponse
from ...types.external_connections.external_connection_phone_number import ExternalConnectionPhoneNumber

__all__ = ["PhoneNumbersResource", "AsyncPhoneNumbersResource"]


class PhoneNumbersResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> PhoneNumbersResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return PhoneNumbersResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> PhoneNumbersResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return PhoneNumbersResourceWithStreamingResponse(self)

    def retrieve(
        self,
        phone_number_id: str,
        *,
        id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PhoneNumberRetrieveResponse:
        """
        Return the details of a phone number associated with the given external
        connection.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        if not phone_number_id:
            raise ValueError(f"Expected a non-empty value for `phone_number_id` but received {phone_number_id!r}")
        return self._get(
            f"/external_connections/{id}/phone_numbers/{phone_number_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PhoneNumberRetrieveResponse,
        )

    def update(
        self,
        phone_number_id: str,
        *,
        id: str,
        location_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PhoneNumberUpdateResponse:
        """
        Asynchronously update settings of the phone number associated with the given
        external connection.

        Args:
          location_id: Identifies the location to assign the phone number to.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        if not phone_number_id:
            raise ValueError(f"Expected a non-empty value for `phone_number_id` but received {phone_number_id!r}")
        return self._patch(
            f"/external_connections/{id}/phone_numbers/{phone_number_id}",
            body=maybe_transform({"location_id": location_id}, phone_number_update_params.PhoneNumberUpdateParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PhoneNumberUpdateResponse,
        )

    def list(
        self,
        id: str,
        *,
        filter: phone_number_list_params.Filter | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncDefaultFlatPagination[ExternalConnectionPhoneNumber]:
        """
        Returns a list of all active phone numbers associated with the given external
        connection.

        Args:
          filter: Filter parameter for phone numbers (deepObject style). Supports filtering by
              phone_number, civic_address_id, and location_id with eq/contains operations.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get_api_list(
            f"/external_connections/{id}/phone_numbers",
            page=SyncDefaultFlatPagination[ExternalConnectionPhoneNumber],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter": filter,
                        "page_number": page_number,
                        "page_size": page_size,
                    },
                    phone_number_list_params.PhoneNumberListParams,
                ),
            ),
            model=ExternalConnectionPhoneNumber,
        )


class AsyncPhoneNumbersResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncPhoneNumbersResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncPhoneNumbersResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncPhoneNumbersResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncPhoneNumbersResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        phone_number_id: str,
        *,
        id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PhoneNumberRetrieveResponse:
        """
        Return the details of a phone number associated with the given external
        connection.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        if not phone_number_id:
            raise ValueError(f"Expected a non-empty value for `phone_number_id` but received {phone_number_id!r}")
        return await self._get(
            f"/external_connections/{id}/phone_numbers/{phone_number_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PhoneNumberRetrieveResponse,
        )

    async def update(
        self,
        phone_number_id: str,
        *,
        id: str,
        location_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PhoneNumberUpdateResponse:
        """
        Asynchronously update settings of the phone number associated with the given
        external connection.

        Args:
          location_id: Identifies the location to assign the phone number to.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        if not phone_number_id:
            raise ValueError(f"Expected a non-empty value for `phone_number_id` but received {phone_number_id!r}")
        return await self._patch(
            f"/external_connections/{id}/phone_numbers/{phone_number_id}",
            body=await async_maybe_transform(
                {"location_id": location_id}, phone_number_update_params.PhoneNumberUpdateParams
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PhoneNumberUpdateResponse,
        )

    def list(
        self,
        id: str,
        *,
        filter: phone_number_list_params.Filter | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[ExternalConnectionPhoneNumber, AsyncDefaultFlatPagination[ExternalConnectionPhoneNumber]]:
        """
        Returns a list of all active phone numbers associated with the given external
        connection.

        Args:
          filter: Filter parameter for phone numbers (deepObject style). Supports filtering by
              phone_number, civic_address_id, and location_id with eq/contains operations.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get_api_list(
            f"/external_connections/{id}/phone_numbers",
            page=AsyncDefaultFlatPagination[ExternalConnectionPhoneNumber],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter": filter,
                        "page_number": page_number,
                        "page_size": page_size,
                    },
                    phone_number_list_params.PhoneNumberListParams,
                ),
            ),
            model=ExternalConnectionPhoneNumber,
        )


class PhoneNumbersResourceWithRawResponse:
    def __init__(self, phone_numbers: PhoneNumbersResource) -> None:
        self._phone_numbers = phone_numbers

        self.retrieve = to_raw_response_wrapper(
            phone_numbers.retrieve,
        )
        self.update = to_raw_response_wrapper(
            phone_numbers.update,
        )
        self.list = to_raw_response_wrapper(
            phone_numbers.list,
        )


class AsyncPhoneNumbersResourceWithRawResponse:
    def __init__(self, phone_numbers: AsyncPhoneNumbersResource) -> None:
        self._phone_numbers = phone_numbers

        self.retrieve = async_to_raw_response_wrapper(
            phone_numbers.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            phone_numbers.update,
        )
        self.list = async_to_raw_response_wrapper(
            phone_numbers.list,
        )


class PhoneNumbersResourceWithStreamingResponse:
    def __init__(self, phone_numbers: PhoneNumbersResource) -> None:
        self._phone_numbers = phone_numbers

        self.retrieve = to_streamed_response_wrapper(
            phone_numbers.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            phone_numbers.update,
        )
        self.list = to_streamed_response_wrapper(
            phone_numbers.list,
        )


class AsyncPhoneNumbersResourceWithStreamingResponse:
    def __init__(self, phone_numbers: AsyncPhoneNumbersResource) -> None:
        self._phone_numbers = phone_numbers

        self.retrieve = async_to_streamed_response_wrapper(
            phone_numbers.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            phone_numbers.update,
        )
        self.list = async_to_streamed_response_wrapper(
            phone_numbers.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = ["SimCardGetDeviceDetailsResponse", "Data"]


class Data(BaseModel):
    brand_name: Optional[str] = None
    """Brand of the device where the SIM card is being used in."""

    device_type: Optional[str] = None
    """Type of the device where the SIM card is being used in."""

    imei: Optional[str] = None
    """IMEI of the device where the SIM card is being used in."""

    device_model_name: Optional[str] = FieldInfo(alias="model_name", default=None)
    """Brand of the device where the SIM card is being used in."""

    operating_system: Optional[str] = None
    """Operating system of the device where the SIM card is being used in."""

    record_type: Optional[str] = None


class SimCardGetDeviceDetailsResponse(BaseModel):
    data: Optional[Data] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .metadata import Metadata as Metadata
from .api_error import APIError as APIError
from .short_code import ShortCode as ShortCode
from .hosted_number import HostedNumber as HostedNumber
from .sim_card_status import SimCardStatus as SimCardStatus
from .simple_sim_card import SimpleSimCard as SimpleSimCard
from .room_participant import RoomParticipant as RoomParticipant
from .porting_order_status import PortingOrderStatus as PortingOrderStatus
from .messaging_feature_set import MessagingFeatureSet as MessagingFeatureSet
from .number_health_metrics import NumberHealthMetrics as NumberHealthMetrics
from .inbound_message_payload import InboundMessagePayload as InboundMessagePayload
from .connection_jitter_buffer import ConnectionJitterBuffer as ConnectionJitterBuffer
from .doc_reqs_requirement_type import DocReqsRequirementType as DocReqsRequirementType
from .messaging_pagination_meta import MessagingPaginationMeta as MessagingPaginationMeta
from .connections_pagination_meta import ConnectionsPaginationMeta as ConnectionsPaginationMeta
from .messaging_hosted_number_order import MessagingHostedNumberOrder as MessagingHostedNumberOrder
from .porting_orders_exception_type import PortingOrdersExceptionType as PortingOrdersExceptionType
from .available_phone_numbers_metadata import AvailablePhoneNumbersMetadata as AvailablePhoneNumbersMetadata
from .connection_noise_suppression_details import ConnectionNoiseSuppressionDetails as ConnectionNoiseSuppressionDetails
from .phone_number_with_messaging_settings import PhoneNumberWithMessagingSettings as PhoneNumberWithMessagingSettings
from .sub_number_order_regulatory_requirement_with_value import (
    SubNumberOrderRegulatoryRequirementWithValue as SubNumberOrderRegulatoryRequirementWithValue,
)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .actions import (
    ActionsResource,
    AsyncActionsResource,
    ActionsResourceWithRawResponse,
    AsyncActionsResourceWithRawResponse,
    ActionsResourceWithStreamingResponse,
    AsyncActionsResourceWithStreamingResponse,
)
from .conferences import (
    ConferencesResource,
    AsyncConferencesResource,
    ConferencesResourceWithRawResponse,
    AsyncConferencesResourceWithRawResponse,
    ConferencesResourceWithStreamingResponse,
    AsyncConferencesResourceWithStreamingResponse,
)

__all__ = [
    "ActionsResource",
    "AsyncActionsResource",
    "ActionsResourceWithRawResponse",
    "AsyncActionsResourceWithRawResponse",
    "ActionsResourceWithStreamingResponse",
    "AsyncActionsResourceWithStreamingResponse",
    "ConferencesResource",
    "AsyncConferencesResource",
    "ConferencesResourceWithRawResponse",
    "AsyncConferencesResourceWithRawResponse",
    "ConferencesResourceWithStreamingResponse",
    "AsyncConferencesResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .run_list_params import RunListParams as RunListParams
from .run_create_params import RunCreateParams as RunCreateParams
from .run_list_response import RunListResponse as RunListResponse
from .run_update_params import RunUpdateParams as RunUpdateParams
from .run_create_response import RunCreateResponse as RunCreateResponse
from .run_update_response import RunUpdateResponse as RunUpdateResponse
from .run_list_runs_params import RunListRunsParams as RunListRunsParams
from .run_retrieve_response import RunRetrieveResponse as RunRetrieveResponse
from .run_list_runs_response import RunListRunsResponse as RunListRunsResponse
from .run_pause_run_response import RunPauseRunResponse as RunPauseRunResponse
from .run_cancel_run_response import RunCancelRunResponse as RunCancelRunResponse
from .run_resume_run_response import RunResumeRunResponse as RunResumeRunResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["RoomUpdateParams"]


class RoomUpdateParams(TypedDict, total=False):
    enable_recording: bool
    """Enable or disable recording for that room."""

    max_participants: int
    """The maximum amount of participants allowed in a room.

    If new participants try to join after that limit is reached, their request will
    be rejected.
    """

    unique_name: str
    """The unique (within the Telnyx account scope) name of the room."""

    webhook_event_failover_url: str
    """
    The failover URL where webhooks related to this room will be sent if sending to
    the primary URL fails. Must include a scheme, such as 'https'.
    """

    webhook_event_url: str
    """The URL where webhooks related to this room will be sent.

    Must include a scheme, such as 'https'.
    """

    webhook_timeout_secs: int
    """Specifies how many seconds to wait before timing out a webhook."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel
from .external_connection_phone_number import ExternalConnectionPhoneNumber

__all__ = ["PhoneNumberUpdateResponse"]


class PhoneNumberUpdateResponse(BaseModel):
    data: Optional[ExternalConnectionPhoneNumber] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel
from .sim_card import SimCard

__all__ = ["SimCardRetrieveResponse"]


class SimCardRetrieveResponse(BaseModel):
    data: Optional[SimCard] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["NumberPoolSettingsParam"]


class NumberPoolSettingsParam(TypedDict, total=False):
    """
    Number Pool allows you to send messages from a pool of numbers of different types, assigning
    weights to each type. The pool consists of all the long code and toll free numbers
    assigned to the messaging profile.

    To disable this feature, set the object field to `null`.
    """

    long_code_weight: Required[float]
    """
    Defines the probability weight for a Long Code number to be selected when
    sending a message. The higher the weight the higher the probability. The sum of
    the weights for all number types does not necessarily need to add to 100. Weight
    must be a non-negative number, and when equal to zero it will remove the number
    type from the pool.
    """

    skip_unhealthy: Required[bool]
    """
    If set to true all unhealthy numbers will be automatically excluded from the
    pool. Health metrics per number are calculated on a regular basis, taking into
    account the deliverability rate and the amount of messages marked as spam by
    upstream carriers. Numbers with a deliverability rate below 25% or spam ratio
    over 75% will be considered unhealthy.
    """

    toll_free_weight: Required[float]
    """
    Defines the probability weight for a Toll Free number to be selected when
    sending a message. The higher the weight the higher the probability. The sum of
    the weights for all number types does not necessarily need to add to 100. Weight
    must be a non-negative number, and when equal to zero it will remove the number
    type from the pool.
    """

    geomatch: bool
    """
    If set to true, Number Pool will try to choose a sending number with the same
    area code as the destination number. If there are no such numbers available, a
    nunber with a different area code will be chosen. Currently only NANP numbers
    are supported.
    """

    sticky_sender: bool
    """
    If set to true, Number Pool will try to choose the same sending number for all
    messages to a particular recipient. If the sending number becomes unhealthy and
    `skip_unhealthy` is set to true, a new number will be chosen.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from typing_extensions import Literal, TypedDict

from ...._types import SequenceNotStr

__all__ = ["ActionMuteParams"]


class ActionMuteParams(TypedDict, total=False):
    exclude: SequenceNotStr[str]
    """List of participant id to exclude from the action."""

    participants: Union[Literal["all"], SequenceNotStr[str]]
    """
    Either a list of participant id to perform the action on, or the keyword "all"
    to perform the action on all participant.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from .._models import BaseModel
from .shared.available_phone_numbers_metadata import AvailablePhoneNumbersMetadata

__all__ = [
    "AvailablePhoneNumberBlockListResponse",
    "Data",
    "DataCostInformation",
    "DataFeature",
    "DataRegionInformation",
]


class DataCostInformation(BaseModel):
    currency: Optional[str] = None
    """The ISO 4217 code for the currency."""

    monthly_cost: Optional[str] = None

    upfront_cost: Optional[str] = None


class DataFeature(BaseModel):
    name: Optional[str] = None


class DataRegionInformation(BaseModel):
    region_name: Optional[str] = None

    region_type: Optional[Literal["country_code", "rate_center", "state", "location"]] = None


class Data(BaseModel):
    cost_information: Optional[DataCostInformation] = None

    features: Optional[List[DataFeature]] = None

    range: Optional[int] = None

    record_type: Optional[Literal["available_phone_number_block"]] = None

    region_information: Optional[List[DataRegionInformation]] = None

    starting_number: Optional[str] = None


class AvailablePhoneNumberBlockListResponse(BaseModel):
    data: Optional[List[Data]] = None

    meta: Optional[AvailablePhoneNumbersMetadata] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from datetime import date
from typing_extensions import Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["SessionList1Params", "Filter", "FilterDateCreatedAt", "FilterDateEndedAt", "FilterDateUpdatedAt"]


class SessionList1Params(TypedDict, total=False):
    filter: Filter
    """Consolidated filter parameter (deepObject style).

    Originally: filter[date_created_at][eq], filter[date_created_at][gte],
    filter[date_created_at][lte], filter[date_updated_at][eq],
    filter[date_updated_at][gte], filter[date_updated_at][lte],
    filter[date_ended_at][eq], filter[date_ended_at][gte],
    filter[date_ended_at][lte], filter[active]
    """

    include_participants: bool
    """To decide if room participants should be included in the response."""

    page_number: Annotated[int, PropertyInfo(alias="page[number]")]

    page_size: Annotated[int, PropertyInfo(alias="page[size]")]


class FilterDateCreatedAt(TypedDict, total=False):
    eq: Annotated[Union[str, date], PropertyInfo(format="iso8601")]
    """ISO 8601 date for filtering room sessions created on that date."""

    gte: Annotated[Union[str, date], PropertyInfo(format="iso8601")]
    """ISO 8601 date for filtering room sessions created on or after that date."""

    lte: Annotated[Union[str, date], PropertyInfo(format="iso8601")]
    """ISO 8601 date for filtering room sessions created on or before that date."""


class FilterDateEndedAt(TypedDict, total=False):
    eq: Annotated[Union[str, date], PropertyInfo(format="iso8601")]
    """ISO 8601 date for filtering room sessions ended on that date."""

    gte: Annotated[Union[str, date], PropertyInfo(format="iso8601")]
    """ISO 8601 date for filtering room sessions ended on or after that date."""

    lte: Annotated[Union[str, date], PropertyInfo(format="iso8601")]
    """ISO 8601 date for filtering room sessions ended on or before that date."""


class FilterDateUpdatedAt(TypedDict, total=False):
    eq: Annotated[Union[str, date], PropertyInfo(format="iso8601")]
    """ISO 8601 date for filtering room sessions updated on that date."""

    gte: Annotated[Union[str, date], PropertyInfo(format="iso8601")]
    """ISO 8601 date for filtering room sessions updated on or after that date."""

    lte: Annotated[Union[str, date], PropertyInfo(format="iso8601")]
    """ISO 8601 date for filtering room sessions updated on or before that date."""


class Filter(TypedDict, total=False):
    """Consolidated filter parameter (deepObject style).

    Originally: filter[date_created_at][eq], filter[date_created_at][gte], filter[date_created_at][lte], filter[date_updated_at][eq], filter[date_updated_at][gte], filter[date_updated_at][lte], filter[date_ended_at][eq], filter[date_ended_at][gte], filter[date_ended_at][lte], filter[active]
    """

    active: bool
    """Filter active or inactive room sessions."""

    date_created_at: FilterDateCreatedAt

    date_ended_at: FilterDateEndedAt

    date_updated_at: FilterDateUpdatedAt



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel
from .wireless_blocklist import WirelessBlocklist

__all__ = ["WirelessBlocklistRetrieveResponse"]


class WirelessBlocklistRetrieveResponse(BaseModel):
    data: Optional[WirelessBlocklist] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel
from ..room_session import RoomSession

__all__ = ["SessionRetrieveResponse"]


class SessionRetrieveResponse(BaseModel):
    data: Optional[RoomSession] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from .osr import (
    OsrResource,
    AsyncOsrResource,
    OsrResourceWithRawResponse,
    AsyncOsrResourceWithRawResponse,
    OsrResourceWithStreamingResponse,
    AsyncOsrResourceWithStreamingResponse,
)
from .usecase import (
    UsecaseResource,
    AsyncUsecaseResource,
    UsecaseResourceWithRawResponse,
    AsyncUsecaseResourceWithRawResponse,
    UsecaseResourceWithStreamingResponse,
    AsyncUsecaseResourceWithStreamingResponse,
)
from ...._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ...._utils import maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....pagination import SyncPerPagePaginationV2, AsyncPerPagePaginationV2
from ...._base_client import AsyncPaginator, make_request_options
from ....types.messaging_10dlc import campaign_list_params, campaign_update_params, campaign_submit_appeal_params
from ....types.messaging_10dlc.telnyx_campaign_csp import TelnyxCampaignCsp
from ....types.messaging_10dlc.campaign_list_response import CampaignListResponse
from ....types.messaging_10dlc.campaign_deactivate_response import CampaignDeactivateResponse
from ....types.messaging_10dlc.campaign_submit_appeal_response import CampaignSubmitAppealResponse
from ....types.messaging_10dlc.campaign_accept_sharing_response import CampaignAcceptSharingResponse
from ....types.messaging_10dlc.campaign_get_mno_metadata_response import CampaignGetMnoMetadataResponse
from ....types.messaging_10dlc.campaign_get_sharing_status_response import CampaignGetSharingStatusResponse
from ....types.messaging_10dlc.campaign_get_operation_status_response import CampaignGetOperationStatusResponse

__all__ = ["CampaignResource", "AsyncCampaignResource"]


class CampaignResource(SyncAPIResource):
    @cached_property
    def usecase(self) -> UsecaseResource:
        return UsecaseResource(self._client)

    @cached_property
    def osr(self) -> OsrResource:
        return OsrResource(self._client)

    @cached_property
    def with_raw_response(self) -> CampaignResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return CampaignResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> CampaignResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return CampaignResourceWithStreamingResponse(self)

    def retrieve(
        self,
        campaign_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TelnyxCampaignCsp:
        """
        Retrieve campaign details by `campaignId`.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not campaign_id:
            raise ValueError(f"Expected a non-empty value for `campaign_id` but received {campaign_id!r}")
        return self._get(
            f"/10dlc/campaign/{campaign_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TelnyxCampaignCsp,
        )

    def update(
        self,
        campaign_id: str,
        *,
        auto_renewal: bool | Omit = omit,
        help_message: str | Omit = omit,
        message_flow: str | Omit = omit,
        reseller_id: str | Omit = omit,
        sample1: str | Omit = omit,
        sample2: str | Omit = omit,
        sample3: str | Omit = omit,
        sample4: str | Omit = omit,
        sample5: str | Omit = omit,
        webhook_failover_url: str | Omit = omit,
        webhook_url: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TelnyxCampaignCsp:
        """Update a campaign's properties by `campaignId`.

        **Please note:** only sample
        messages are editable.

        Args:
          auto_renewal: Help message of the campaign.

          help_message: Help message of the campaign.

          message_flow: Message flow description.

          reseller_id: Alphanumeric identifier of the reseller that you want to associate with this
              campaign.

          sample1: Message sample. Some campaign tiers require 1 or more message samples.

          sample2: Message sample. Some campaign tiers require 2 or more message samples.

          sample3: Message sample. Some campaign tiers require 3 or more message samples.

          sample4: Message sample. Some campaign tiers require 4 or more message samples.

          sample5: Message sample. Some campaign tiers require 5 or more message samples.

          webhook_failover_url: Webhook failover to which campaign status updates are sent.

          webhook_url: Webhook to which campaign status updates are sent.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not campaign_id:
            raise ValueError(f"Expected a non-empty value for `campaign_id` but received {campaign_id!r}")
        return self._put(
            f"/10dlc/campaign/{campaign_id}",
            body=maybe_transform(
                {
                    "auto_renewal": auto_renewal,
                    "help_message": help_message,
                    "message_flow": message_flow,
                    "reseller_id": reseller_id,
                    "sample1": sample1,
                    "sample2": sample2,
                    "sample3": sample3,
                    "sample4": sample4,
                    "sample5": sample5,
                    "webhook_failover_url": webhook_failover_url,
                    "webhook_url": webhook_url,
                },
                campaign_update_params.CampaignUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TelnyxCampaignCsp,
        )

    def list(
        self,
        *,
        brand_id: str,
        page: int | Omit = omit,
        records_per_page: int | Omit = omit,
        sort: Literal[
            "assignedPhoneNumbersCount",
            "-assignedPhoneNumbersCount",
            "campaignId",
            "-campaignId",
            "createdAt",
            "-createdAt",
            "status",
            "-status",
            "tcrCampaignId",
            "-tcrCampaignId",
        ]
        | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncPerPagePaginationV2[CampaignListResponse]:
        """
        Retrieve a list of campaigns associated with a supplied `brandId`.

        Args:
          page: The 1-indexed page number to get. The default value is `1`.

          records_per_page: The amount of records per page, limited to between 1 and 500 inclusive. The
              default value is `10`.

          sort: Specifies the sort order for results. If not given, results are sorted by
              createdAt in descending order.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/10dlc/campaign",
            page=SyncPerPagePaginationV2[CampaignListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "brand_id": brand_id,
                        "page": page,
                        "records_per_page": records_per_page,
                        "sort": sort,
                    },
                    campaign_list_params.CampaignListParams,
                ),
            ),
            model=CampaignListResponse,
        )

    def accept_sharing(
        self,
        campaign_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CampaignAcceptSharingResponse:
        """
        Manually accept a campaign shared with Telnyx

        Args:
          campaign_id: TCR's ID for the campaign to import

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not campaign_id:
            raise ValueError(f"Expected a non-empty value for `campaign_id` but received {campaign_id!r}")
        return self._post(
            f"/10dlc/campaign/acceptSharing/{campaign_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CampaignAcceptSharingResponse,
        )

    def deactivate(
        self,
        campaign_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CampaignDeactivateResponse:
        """Terminate a campaign.

        Note that once deactivated, a campaign cannot be restored.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not campaign_id:
            raise ValueError(f"Expected a non-empty value for `campaign_id` but received {campaign_id!r}")
        return self._delete(
            f"/10dlc/campaign/{campaign_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CampaignDeactivateResponse,
        )

    def get_mno_metadata(
        self,
        campaign_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CampaignGetMnoMetadataResponse:
        """
        Get the campaign metadata for each MNO it was submitted to.

        Args:
          campaign_id: ID of the campaign in question

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not campaign_id:
            raise ValueError(f"Expected a non-empty value for `campaign_id` but received {campaign_id!r}")
        return self._get(
            f"/10dlc/campaign/{campaign_id}/mnoMetadata",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CampaignGetMnoMetadataResponse,
        )

    def get_operation_status(
        self,
        campaign_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CampaignGetOperationStatusResponse:
        """
        Retrieve campaign's operation status at MNO level.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not campaign_id:
            raise ValueError(f"Expected a non-empty value for `campaign_id` but received {campaign_id!r}")
        return self._get(
            f"/10dlc/campaign/{campaign_id}/operationStatus",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CampaignGetOperationStatusResponse,
        )

    def get_sharing_status(
        self,
        campaign_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CampaignGetSharingStatusResponse:
        """
        Get Sharing Status

        Args:
          campaign_id: ID of the campaign in question

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not campaign_id:
            raise ValueError(f"Expected a non-empty value for `campaign_id` but received {campaign_id!r}")
        return self._get(
            f"/10dlc/campaign/{campaign_id}/sharing",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CampaignGetSharingStatusResponse,
        )

    def submit_appeal(
        self,
        campaign_id: str,
        *,
        appeal_reason: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CampaignSubmitAppealResponse:
        """
        Submits an appeal for rejected native campaigns in TELNYX_FAILED or MNO_REJECTED
        status. The appeal is recorded for manual compliance team review and the
        campaign status is reset to TCR_ACCEPTED. Note: Appeal forwarding is handled
        manually to allow proper review before incurring upstream charges.

        Args:
          appeal_reason: Detailed explanation of why the campaign should be reconsidered and what changes
              have been made to address the rejection reason.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not campaign_id:
            raise ValueError(f"Expected a non-empty value for `campaign_id` but received {campaign_id!r}")
        return self._post(
            f"/10dlc/campaign/{campaign_id}/appeal",
            body=maybe_transform(
                {"appeal_reason": appeal_reason}, campaign_submit_appeal_params.CampaignSubmitAppealParams
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CampaignSubmitAppealResponse,
        )


class AsyncCampaignResource(AsyncAPIResource):
    @cached_property
    def usecase(self) -> AsyncUsecaseResource:
        return AsyncUsecaseResource(self._client)

    @cached_property
    def osr(self) -> AsyncOsrResource:
        return AsyncOsrResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncCampaignResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncCampaignResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncCampaignResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncCampaignResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        campaign_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TelnyxCampaignCsp:
        """
        Retrieve campaign details by `campaignId`.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not campaign_id:
            raise ValueError(f"Expected a non-empty value for `campaign_id` but received {campaign_id!r}")
        return await self._get(
            f"/10dlc/campaign/{campaign_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TelnyxCampaignCsp,
        )

    async def update(
        self,
        campaign_id: str,
        *,
        auto_renewal: bool | Omit = omit,
        help_message: str | Omit = omit,
        message_flow: str | Omit = omit,
        reseller_id: str | Omit = omit,
        sample1: str | Omit = omit,
        sample2: str | Omit = omit,
        sample3: str | Omit = omit,
        sample4: str | Omit = omit,
        sample5: str | Omit = omit,
        webhook_failover_url: str | Omit = omit,
        webhook_url: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TelnyxCampaignCsp:
        """Update a campaign's properties by `campaignId`.

        **Please note:** only sample
        messages are editable.

        Args:
          auto_renewal: Help message of the campaign.

          help_message: Help message of the campaign.

          message_flow: Message flow description.

          reseller_id: Alphanumeric identifier of the reseller that you want to associate with this
              campaign.

          sample1: Message sample. Some campaign tiers require 1 or more message samples.

          sample2: Message sample. Some campaign tiers require 2 or more message samples.

          sample3: Message sample. Some campaign tiers require 3 or more message samples.

          sample4: Message sample. Some campaign tiers require 4 or more message samples.

          sample5: Message sample. Some campaign tiers require 5 or more message samples.

          webhook_failover_url: Webhook failover to which campaign status updates are sent.

          webhook_url: Webhook to which campaign status updates are sent.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not campaign_id:
            raise ValueError(f"Expected a non-empty value for `campaign_id` but received {campaign_id!r}")
        return await self._put(
            f"/10dlc/campaign/{campaign_id}",
            body=await async_maybe_transform(
                {
                    "auto_renewal": auto_renewal,
                    "help_message": help_message,
                    "message_flow": message_flow,
                    "reseller_id": reseller_id,
                    "sample1": sample1,
                    "sample2": sample2,
                    "sample3": sample3,
                    "sample4": sample4,
                    "sample5": sample5,
                    "webhook_failover_url": webhook_failover_url,
                    "webhook_url": webhook_url,
                },
                campaign_update_params.CampaignUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TelnyxCampaignCsp,
        )

    def list(
        self,
        *,
        brand_id: str,
        page: int | Omit = omit,
        records_per_page: int | Omit = omit,
        sort: Literal[
            "assignedPhoneNumbersCount",
            "-assignedPhoneNumbersCount",
            "campaignId",
            "-campaignId",
            "createdAt",
            "-createdAt",
            "status",
            "-status",
            "tcrCampaignId",
            "-tcrCampaignId",
        ]
        | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[CampaignListResponse, AsyncPerPagePaginationV2[CampaignListResponse]]:
        """
        Retrieve a list of campaigns associated with a supplied `brandId`.

        Args:
          page: The 1-indexed page number to get. The default value is `1`.

          records_per_page: The amount of records per page, limited to between 1 and 500 inclusive. The
              default value is `10`.

          sort: Specifies the sort order for results. If not given, results are sorted by
              createdAt in descending order.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/10dlc/campaign",
            page=AsyncPerPagePaginationV2[CampaignListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "brand_id": brand_id,
                        "page": page,
                        "records_per_page": records_per_page,
                        "sort": sort,
                    },
                    campaign_list_params.CampaignListParams,
                ),
            ),
            model=CampaignListResponse,
        )

    async def accept_sharing(
        self,
        campaign_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CampaignAcceptSharingResponse:
        """
        Manually accept a campaign shared with Telnyx

        Args:
          campaign_id: TCR's ID for the campaign to import

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not campaign_id:
            raise ValueError(f"Expected a non-empty value for `campaign_id` but received {campaign_id!r}")
        return await self._post(
            f"/10dlc/campaign/acceptSharing/{campaign_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CampaignAcceptSharingResponse,
        )

    async def deactivate(
        self,
        campaign_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CampaignDeactivateResponse:
        """Terminate a campaign.

        Note that once deactivated, a campaign cannot be restored.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not campaign_id:
            raise ValueError(f"Expected a non-empty value for `campaign_id` but received {campaign_id!r}")
        return await self._delete(
            f"/10dlc/campaign/{campaign_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CampaignDeactivateResponse,
        )

    async def get_mno_metadata(
        self,
        campaign_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CampaignGetMnoMetadataResponse:
        """
        Get the campaign metadata for each MNO it was submitted to.

        Args:
          campaign_id: ID of the campaign in question

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not campaign_id:
            raise ValueError(f"Expected a non-empty value for `campaign_id` but received {campaign_id!r}")
        return await self._get(
            f"/10dlc/campaign/{campaign_id}/mnoMetadata",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CampaignGetMnoMetadataResponse,
        )

    async def get_operation_status(
        self,
        campaign_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CampaignGetOperationStatusResponse:
        """
        Retrieve campaign's operation status at MNO level.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not campaign_id:
            raise ValueError(f"Expected a non-empty value for `campaign_id` but received {campaign_id!r}")
        return await self._get(
            f"/10dlc/campaign/{campaign_id}/operationStatus",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CampaignGetOperationStatusResponse,
        )

    async def get_sharing_status(
        self,
        campaign_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CampaignGetSharingStatusResponse:
        """
        Get Sharing Status

        Args:
          campaign_id: ID of the campaign in question

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not campaign_id:
            raise ValueError(f"Expected a non-empty value for `campaign_id` but received {campaign_id!r}")
        return await self._get(
            f"/10dlc/campaign/{campaign_id}/sharing",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CampaignGetSharingStatusResponse,
        )

    async def submit_appeal(
        self,
        campaign_id: str,
        *,
        appeal_reason: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CampaignSubmitAppealResponse:
        """
        Submits an appeal for rejected native campaigns in TELNYX_FAILED or MNO_REJECTED
        status. The appeal is recorded for manual compliance team review and the
        campaign status is reset to TCR_ACCEPTED. Note: Appeal forwarding is handled
        manually to allow proper review before incurring upstream charges.

        Args:
          appeal_reason: Detailed explanation of why the campaign should be reconsidered and what changes
              have been made to address the rejection reason.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not campaign_id:
            raise ValueError(f"Expected a non-empty value for `campaign_id` but received {campaign_id!r}")
        return await self._post(
            f"/10dlc/campaign/{campaign_id}/appeal",
            body=await async_maybe_transform(
                {"appeal_reason": appeal_reason}, campaign_submit_appeal_params.CampaignSubmitAppealParams
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CampaignSubmitAppealResponse,
        )


class CampaignResourceWithRawResponse:
    def __init__(self, campaign: CampaignResource) -> None:
        self._campaign = campaign

        self.retrieve = to_raw_response_wrapper(
            campaign.retrieve,
        )
        self.update = to_raw_response_wrapper(
            campaign.update,
        )
        self.list = to_raw_response_wrapper(
            campaign.list,
        )
        self.accept_sharing = to_raw_response_wrapper(
            campaign.accept_sharing,
        )
        self.deactivate = to_raw_response_wrapper(
            campaign.deactivate,
        )
        self.get_mno_metadata = to_raw_response_wrapper(
            campaign.get_mno_metadata,
        )
        self.get_operation_status = to_raw_response_wrapper(
            campaign.get_operation_status,
        )
        self.get_sharing_status = to_raw_response_wrapper(
            campaign.get_sharing_status,
        )
        self.submit_appeal = to_raw_response_wrapper(
            campaign.submit_appeal,
        )

    @cached_property
    def usecase(self) -> UsecaseResourceWithRawResponse:
        return UsecaseResourceWithRawResponse(self._campaign.usecase)

    @cached_property
    def osr(self) -> OsrResourceWithRawResponse:
        return OsrResourceWithRawResponse(self._campaign.osr)


class AsyncCampaignResourceWithRawResponse:
    def __init__(self, campaign: AsyncCampaignResource) -> None:
        self._campaign = campaign

        self.retrieve = async_to_raw_response_wrapper(
            campaign.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            campaign.update,
        )
        self.list = async_to_raw_response_wrapper(
            campaign.list,
        )
        self.accept_sharing = async_to_raw_response_wrapper(
            campaign.accept_sharing,
        )
        self.deactivate = async_to_raw_response_wrapper(
            campaign.deactivate,
        )
        self.get_mno_metadata = async_to_raw_response_wrapper(
            campaign.get_mno_metadata,
        )
        self.get_operation_status = async_to_raw_response_wrapper(
            campaign.get_operation_status,
        )
        self.get_sharing_status = async_to_raw_response_wrapper(
            campaign.get_sharing_status,
        )
        self.submit_appeal = async_to_raw_response_wrapper(
            campaign.submit_appeal,
        )

    @cached_property
    def usecase(self) -> AsyncUsecaseResourceWithRawResponse:
        return AsyncUsecaseResourceWithRawResponse(self._campaign.usecase)

    @cached_property
    def osr(self) -> AsyncOsrResourceWithRawResponse:
        return AsyncOsrResourceWithRawResponse(self._campaign.osr)


class CampaignResourceWithStreamingResponse:
    def __init__(self, campaign: CampaignResource) -> None:
        self._campaign = campaign

        self.retrieve = to_streamed_response_wrapper(
            campaign.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            campaign.update,
        )
        self.list = to_streamed_response_wrapper(
            campaign.list,
        )
        self.accept_sharing = to_streamed_response_wrapper(
            campaign.accept_sharing,
        )
        self.deactivate = to_streamed_response_wrapper(
            campaign.deactivate,
        )
        self.get_mno_metadata = to_streamed_response_wrapper(
            campaign.get_mno_metadata,
        )
        self.get_operation_status = to_streamed_response_wrapper(
            campaign.get_operation_status,
        )
        self.get_sharing_status = to_streamed_response_wrapper(
            campaign.get_sharing_status,
        )
        self.submit_appeal = to_streamed_response_wrapper(
            campaign.submit_appeal,
        )

    @cached_property
    def usecase(self) -> UsecaseResourceWithStreamingResponse:
        return UsecaseResourceWithStreamingResponse(self._campaign.usecase)

    @cached_property
    def osr(self) -> OsrResourceWithStreamingResponse:
        return OsrResourceWithStreamingResponse(self._campaign.osr)


class AsyncCampaignResourceWithStreamingResponse:
    def __init__(self, campaign: AsyncCampaignResource) -> None:
        self._campaign = campaign

        self.retrieve = async_to_streamed_response_wrapper(
            campaign.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            campaign.update,
        )
        self.list = async_to_streamed_response_wrapper(
            campaign.list,
        )
        self.accept_sharing = async_to_streamed_response_wrapper(
            campaign.accept_sharing,
        )
        self.deactivate = async_to_streamed_response_wrapper(
            campaign.deactivate,
        )
        self.get_mno_metadata = async_to_streamed_response_wrapper(
            campaign.get_mno_metadata,
        )
        self.get_operation_status = async_to_streamed_response_wrapper(
            campaign.get_operation_status,
        )
        self.get_sharing_status = async_to_streamed_response_wrapper(
            campaign.get_sharing_status,
        )
        self.submit_appeal = async_to_streamed_response_wrapper(
            campaign.submit_appeal,
        )

    @cached_property
    def usecase(self) -> AsyncUsecaseResourceWithStreamingResponse:
        return AsyncUsecaseResourceWithStreamingResponse(self._campaign.usecase)

    @cached_property
    def osr(self) -> AsyncOsrResourceWithStreamingResponse:
        return AsyncOsrResourceWithStreamingResponse(self._campaign.osr)
